@@ -4,7 +4,7 @@
 # SOAP::Lite is free software; you can redistribute it
 # and/or modify it under the same terms as Perl itself.
 #
-# $Id: SOAP::Transport::POP3.pm,v 0.40 2000/10/15 18:20:55 $
+# $Id: SOAP::Transport::POP3.pm,v 0.41 2000/10/31 01:24:51 $
 #
 # ======================================================================
 
@@ -12,7 +12,11 @@ package SOAP::Transport::POP3;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.40';
+$VERSION = '0.41';
+
+use Net::POP3; 
+use MIME::Parser; 
+use URI::Escape; 
 
 # ======================================================================
 
@@ -23,24 +27,17 @@ use SOAP::Lite;
 use vars qw(@ISA $AUTOLOAD);
 @ISA = qw(SOAP::Server);
 
-sub new { eval "use Net::POP3; use MIME::Parser; use URI::Escape"; die if $@;
+sub new {
   my $self = shift;
   my $class = ref($self) || $self;
     
   unless (ref $self) {
-    $self = bless $class->SUPER::new() => $class;
     my($server, $auth) = reverse split /@/, URI::Escape::uri_unescape(shift);
+    $self = $class->SUPER::new(@_);
     $self->{_pop3server} = Net::POP3->new($server) or croak "Can't connect to $server: $!";
     $self->{_pop3server}->login(split /:/, $auth) or croak "Can't authenticate to $server"
       if defined $auth;
   }
-
-  if (@_) {
-    my %parameters = @_;
-    foreach (grep {defined $parameters{$_}} keys %parameters) {
-      $self->$_($parameters{$_}) if $self->can($_);
-    }
-  }
   return $self;
 }
 
